use std::sync::Arc;

use async_trait::async_trait;
use chrono::{DateTime, Utc};
use sqlx::{Pool, Postgres};

use crate::{
    adapter::gateway::post_db_service::PostDbService, application::error::post_error::PostError,
    domain::entity::post_info::PostInfo,
};

pub struct PostDbServiceImpl {
    db_pool: Arc<Pool<Postgres>>,
}

impl PostDbServiceImpl {
    pub fn new(db_pool: Arc<Pool<Postgres>>) -> Self {
        Self { db_pool }
    }
}

#[async_trait]
impl PostDbService for PostDbServiceImpl {
    async fn get_all_post_info(&self) -> Result<Vec<PostInfo>, PostError> {
        let posts = sqlx::query!(
            r#"
            SELECT 
                p.id, 
                p.title, 
                p.description, 
                p.preview_image_url,
                p.published_time
            FROM post p
            WHERE p.deleted_time IS NULL
            "#
        )
        .fetch_all(&*self.db_pool)
        .await
        .map_err(|err| PostError::DatabaseError(err.to_string()))?;

        let mut post_infos = Vec::new();

        for post in posts {
            let labels = sqlx::query!(
                r#"
                SELECT 
                    l.id,
                    l.name,
                    l.color
                FROM label l
                JOIN post_label pl ON l.id = pl.label_id
                WHERE pl.post_id = $1
                AND l.deleted_time IS NULL
                "#,
                post.id
            )
            .fetch_all(&*self.db_pool)
            .await
            .map_err(|err| PostError::DatabaseError(err.to_string()))?;

            let domain_labels = labels
                .into_iter()
                .map(|label| crate::domain::entity::label::Label {
                    id: label.id,
                    name: label.name,
                    color: label.color as u32,
                })
                .collect();

            post_infos.push(PostInfo {
                id: post.id,
                title: post.title,
                description: post.description,
                preview_image_url: post.preview_image_url,
                labels: domain_labels,
                published_time: post
                    .published_time
                    .map(|dt| DateTime::<Utc>::from_naive_utc_and_offset(dt, Utc)),
            });
        }

        Ok(post_infos)
    }
}
